unit UAutoCAD;

interface

uses
  Windows, SysUtils, Messages, Variants, Classes, Registry, ComObj, ActiveX,
  Graphics, AutoCAD_TLB;

resourcestring
  rsDefClassName = 'AutoCAD.Application';

type
  TPoint3D = record
    X, Y, Z: Double;
  end;

  TLayerState = (lsLayerOn, lsFreeze, lsLock, lsPlottable);
  TLayerStates = set of TLayerState;

  procedure AcConnect(const acClassName: String; const acNewInstance: Boolean;
      const acVisible: Boolean);
  procedure AcShow(WndState: AcWindowState = acMax);
  procedure AcHide;
  procedure AcRelease;
  function EnumAutoCAD(Items: TStrings): Boolean;
  function GetCurrAutoCADClassName: String;
  function AutoCADFamilyNameToClassName(const AFamilyName: String): String;
  function EnumAutoCADHardcopy(Items: TStrings): Boolean;
  //
  function EnumAutoCADDocuments(Items: TStrings): Integer;
  function EnumAutoCADLayers(Items: TStrings): Integer;
  function EnumAutoCADTextStyles(Items: TStrings): Integer;
  function EnumAutoCADDimStyles(Items: TStrings): Integer;
  function EnumAutoCADLinetypes(Items: TStrings): Integer;
  //
  function LayerExist(const ALayerName: String): Boolean;
  function LinetypeExist(const ALinetypeName: String): Boolean;
  //
  procedure SetActiveLayer(AName: String);
  //
  function Point3D(X, Y: Double; Z: Double = 0): TPoint3D;
  function AcadPoint(X, Y: Double; Z: Double = 0): OleVariant; overload;
  function AcadPoint(APoint: TPoint3D): OleVariant; overload;
  function AcadColorToColor(AColor: AcColor): TColor;

const
  AcColorTable: array [0..254] of TColor = (
    $FF,     $FFFF,   $FF00,   $FFFF00, $FF0000, $FF00FF, $0,      $4C4C4C,
    $A5A5A5, $FF,     $7F7FFF, $A5,     $5252A5, $7F,     $3F3F7F, $4C,
    $26264C, $26,     $131326, $3FFF,   $7F9FFF, $29A5,   $5267A5, $1F7F,
    $3F4F7F, $134C,   $262F4C, $926,    $131726, $7FFF,   $7FBFFF, $52A5,
    $527CA5, $3F7F,   $3F5F7F, $264C,   $26394C, $1326,   $131C26, $BFFF,
    $7FDFFF, $7CA5,   $5291A5, $5F7F,   $3F6F7F, $394C,   $26424C, $1C26,
    $132126, $FFFF,   $7FFFFF, $A5A5,   $52A5A5, $7F7F,   $3F7F7F, $4C4C,
    $264C4C, $2626,   $132626, $FFBF,   $7FFFDF, $A57C,   $52A591, $7F5F,
    $3F7F6F, $4C39,   $264C42, $261C,   $132621, $FF7F,   $7FFFBF, $A552,
    $52A57C, $7F3F,   $3F7F5F, $4C26,   $264C39, $2613,   $13261C, $FF3F,
    $7FFF9F, $A529,   $52A567, $7F1F,   $3F7F4F, $4C13,   $264C2F, $2609,
    $132617, $FF00,   $7FFF7F, $A500,   $52A552, $7F00,   $3F7F3F, $4C00,
    $264C26, $2600,   $132613, $3FFF00, $9FFF7F, $29A500, $67A552, $1F7F00,
    $4F7F3F, $134C00, $2F4C26, $92600,  $172613, $7FFF00, $BFFF7F, $52A500,
    $7CA552, $3F7F00, $5F7F3F, $264C00, $394C26, $132600, $1C2613, $BFFF00,
    $DFFF7F, $7CA500, $91A552, $5F7F00, $6F7F3F, $394C00, $424C26, $1C2600,
    $212613, $FFFF00, $FFFF7F, $A5A500, $A5A552, $7F7F00, $7F7F3F, $4C4C00,
    $4C4C26, $262600, $262613, $FFBF00, $FFDF7F, $A57C00, $A59152, $7F5F00,
    $7F6F3F, $4C3900, $4C4226, $261C00, $262113, $FF7F00, $FFBF7F, $A55200,
    $A57C52, $7F3F00, $7F5F3F, $4C2600, $4C3926, $261300, $261C13, $FF3F00,
    $FF9F7F, $A52900, $A56752, $7F1F00, $7F4F3F, $4C1300, $4C2F26, $260900,
    $261713, $FF0000, $FF7F7F, $A50000, $A55252, $7F0000, $7F3F3F, $4C0000,
    $4C2626, $260000, $261313, $FF003F, $FF7F9F, $A50029, $A55267, $7F001F,
    $7F3F4F, $4C0013, $4C262F, $260009, $261317, $FF007F, $FF7FBF, $A50052,
    $A5527C, $7F003F, $7F3F5F, $4C0026, $4C2639, $260013, $26131C, $FF00BF,
    $FF7FDF, $A5007C, $A55291, $7F005F, $7F3F6F, $4C0039, $4C2642, $26001C,
    $261321, $FF00FF, $FF7FFF, $A500A5, $A552A5, $7F007F, $7F3F7F, $4C004C,
    $4C264C, $260026, $261326, $BF00FF, $DF7FFF, $7C00A5, $9152A5, $5F007F,
    $6F3F7F, $39004C, $42264C, $1C0026, $211326, $7F00FF, $BF7FFF, $5200A5,
    $7C52A5, $3F007F, $5F3F7F, $26004C, $39264C, $130026, $1C1326, $3F00FF,
    $9F7FFF, $2900A5, $6752A5, $1F007F, $4F3F7F, $13004C, $2F264C, $90026,
    $171326, $545454, $767676, $989898, $BABABA, $DCDCDC, $FFFFFF);
    
var
  Acad: OleVariant;
  
implementation

resourcestring
  rsAcadNotFound = 'AutoCAD  !';
  rsAutoCADRegPath = '\SOFTWARE\Autodesk\AutoCAD';

procedure AcConnect(const acClassName: String; const acNewInstance: Boolean;
    const acVisible: Boolean);
var
  IU: IUnknown;
  ClassID: TCLSID;
  IsCreate: Boolean;
  ErrorCode: HResult;
begin
  if VarIsClear(Acad) then //     
  begin
    //  AutoCAD     
    ErrorCode:= CLSIDFromProgID(PWideChar(WideString(acClassName)), ClassID);
    if not Succeeded(ErrorCode) then
      raise EOleSysError.Create(rsAcadNotFound, ErrorCode, 0);
    //       AutoCAD
    IsCreate:= acNewInstance or (not Succeeded(GetActiveObject(ClassID, nil, IU)));
    if IsCreate then
      try
        //    AutoCAD
        Acad:= CreateComObject(ClassID) as IDispatch;
      except
        raise EOleSysError.Create(rsAcadNotFound, ErrorCode, 0);
      end
    else
      Acad:= IU as IDispatch;
    //   AutoCAD
    if not VarIsClear(Acad) then Acad.Visible:= acVisible;
  end;
end;

procedure AcShow(WndState: AcWindowState = acMax);
begin
  if not VarIsClear(Acad) then
  begin
    Acad.Visible:= True;
    Acad.WindowState:= WndState;
  end;
end;

procedure AcHide;
begin
  if not VarIsClear(Acad) then Acad.Visible:= False;
end;

procedure AcRelease;
begin
  if not VarIsClear(Acad) then
  begin
    Acad.Quit;
    Acad:= Unassigned;
  end;
end;

function EnumAutoCAD(Items: TStrings): Boolean;
var
  reg: TRegistry;
begin
  reg:= TRegistry.Create;
  try
    with reg do
    begin
      RootKey:= HKEY_LOCAL_MACHINE;
      Result:= OpenKeyReadOnly(rsAutoCADRegPath);
      if not Result then Exit;
      GetKeyNames(Items);
    end;
  finally
    FreeAndNil(reg);
  end;
end;

function GetCurrAutoCADClassName: String;
var
  reg: TRegistry;
  res: Boolean;
begin
  Result:= EmptyStr;
  reg:= TRegistry.Create;
  try
    with reg do
    begin
      RootKey:= HKEY_CURRENT_USER;
      res:= OpenKeyReadOnly('\Software\Classes\' + rsDefClassName + '\CurVer');
      if not res then Exit;
      Result:= ReadString('');
    end;
  finally
    FreeAndNil(reg);
  end;
end;

function AutoCADFamilyNameToClassName(const AFamilyName: String): String;
var
  p: Integer;
begin
  Result:= AFamilyName;
  p:= Pos('.', Result);
  if Copy(Result, p, MaxInt) = '.0' then
    Delete(Result, p, MaxInt);
  Result:= StringReplace(Result, 'R', rsDefClassName + '.', [rfIgnoreCase]);
end;

function EnumAutoCADHardcopy(Items: TStrings): Boolean;
var
  reg: TRegistry;
begin
  reg:= TRegistry.Create;
  try
    with reg do
    begin
      RootKey:= HKEY_LOCAL_MACHINE;
      Result:= OpenKeyReadOnly('SOFTWARE\Autodesk\Hardcopy');
      if not Result then Exit;
      GetValueNames(Items);
    end;
  finally
    FreeAndNil(reg);
  end;
end;

function EnumAutoCADDocuments(Items: TStrings): Integer;
var
  i, cnt: Integer;
  ActiveDocName, DocName: String;
begin
  Result:= -1;
  if not VarIsClear(Acad) then
  begin
    ActiveDocName:= Acad.ActiveDocument.Name;
    cnt:= Acad.Documents.Count;
    with Items do
    begin
      BeginUpdate;
      try
        Clear;
        for i:= 0 to cnt - 1 do
        begin
          DocName:= Acad.Documents.Item(i).Name;
          Add(DocName);
          if AnsiCompareText(DocName, ActiveDocName) = 0 then
            Result:= i;
        end;
      finally
        EndUpdate;
      end;
    end;
  end;
end;

function EnumAutoCADLayers(Items: TStrings): Integer;
var
  AcadDocument, Layer: OleVariant;
  ActiveLayerName, LayerName: String;
  i, LayersCount, Tmp: Integer;
  States: TLayerStates;
begin
  Result:= -1;
  if not VarIsClear(Acad) then
  begin
    AcadDocument:= Acad.ActiveDocument;
    with Items do
    begin
      BeginUpdate;
      try
        Clear;
        ActiveLayerName:= AcadDocument.ActiveLayer.Name;
        LayersCount:= AcadDocument.Layers.Count;
        for i:= 0 to LayersCount - 1 do
        begin
          Layer:= AcadDocument.Layers.Item(i);
          States:= [];
          if Layer.LayerOn then States:= States + [lsLayerOn];
          if Layer.Freeze then States:= States + [lsFreeze];
          if Layer.Lock then States:= States + [lsLock];
          if Layer.Plottable then States:= States + [lsPlottable];
          LayerName:= Layer.Name;
          System.Move(States, Tmp, 1);
          AddObject(LayerName, TObject(Tmp));
          if AnsiCompareText(LayerName, ActiveLayerName) = 0 then
            Result:= i;
        end;
      finally
        EndUpdate;
      end;
    end;
  end;
end;

function EnumAutoCADTextStyles(Items: TStrings): Integer;
var
  AcadDocument: OleVariant;
  ActiveStyleName, StyleName: String;
  i, StylesCount: Integer;
begin
  Result:= -1;
  if not VarIsClear(Acad) then
  begin
    AcadDocument:= Acad.ActiveDocument;
    with Items do
    begin
      BeginUpdate;
      try
        Clear;
        ActiveStyleName:= AcadDocument.ActiveTextStyle.Name;
        StylesCount:= AcadDocument.TextStyles.Count;
        for i:= 0 to StylesCount - 1 do
        begin
          StyleName:= AcadDocument.TextStyles.Item(i).Name;
          Add(StyleName);
          if AnsiCompareText(StyleName, ActiveStyleName) = 0 then
            Result:= i;
        end;
      finally
        EndUpdate;
      end;
    end;
  end;
end;

function EnumAutoCADDimStyles(Items: TStrings): Integer;
var
  AcadDocument: OleVariant;
  ActiveStyleName, StyleName: String;
  i, StylesCount: Integer;
begin
  Result:= -1;
  if not VarIsClear(Acad) then
  begin
    AcadDocument:= Acad.ActiveDocument;
    with Items do
    begin
      BeginUpdate;
      try
        Clear;
        ActiveStyleName:= AcadDocument.ActiveDimStyle.Name;
        StylesCount:= AcadDocument.DimStyles.Count;
        for i:= 0 to StylesCount - 1 do
        begin
          StyleName:= AcadDocument.DimStyles.Item(i).Name;
          Add(StyleName);
          if AnsiCompareText(StyleName, ActiveStyleName) = 0 then
            Result:= i;
        end;
      finally
        EndUpdate;
      end;
    end;
  end;
end;

function EnumAutoCADLinetypes(Items: TStrings): Integer;
var
  AcadDocument: OleVariant;
  ActiveLinetype, LinetypeName: String;
  i, LinetypesCount: Integer;
begin
  Result:= -1;
  if not VarIsClear(Acad) then
  begin
    AcadDocument:= Acad.ActiveDocument;
    with Items do
    begin
      BeginUpdate;
      try
        Clear;
        ActiveLinetype:= AcadDocument.ActiveLinetype.Name;
        LinetypesCount:= AcadDocument.Linetypes.Count;
        for i:= 0 to LinetypesCount - 1 do
        begin
          LinetypeName:= AcadDocument.Linetypes.Item(i).Name;
          Add(LinetypeName);
          if AnsiCompareText(LinetypeName, ActiveLinetype) = 0 then
            Result:= i;
        end;
      finally
        EndUpdate;
      end;
    end;
  end;
end;

function LayerExist(const ALayerName: String): Boolean;
var
  AcadDocument: OleVariant;
  i, cnt: Integer;
begin
  Result:= False;
  if not VarIsClear(Acad) then
  begin
    AcadDocument:= Acad.ActiveDocument;
    cnt:= AcadDocument.Layers.Count;
    for i:= 0 to cnt - 1 do
    begin
      if AcadDocument.Layers.Item(i).Name = ALayerName then
      begin
        Result:= True;
        Break;
      end;
    end;
  end;
end;

function LinetypeExist(const ALinetypeName: String): Boolean;
var
  AcadDocument: OleVariant;
  i, cnt: Integer;
begin
  Result:= False;
  if not VarIsClear(Acad) then
  begin
    AcadDocument:= Acad.ActiveDocument;
    cnt:= AcadDocument.Linetypes.Count;
    for i:= 0 to cnt - 1 do
    begin
      if AcadDocument.Linetypes.Item(i).Name = ALinetypeName then
      begin
        Result:= True;
        Break;
      end;
    end;
  end;
end;

procedure SetActiveLayer(AName: String);
var
  AcadDocument, Layer: OleVariant;
  ActiveLayerName: String;
  i, LayersCount: Integer;
begin
  if not VarIsClear(Acad) then
  begin
    AcadDocument:= Acad.ActiveDocument;
    ActiveLayerName:= AcadDocument.ActiveLayer.Name;
    if AnsiCompareText(AName, ActiveLayerName) = 0 then Exit;
    LayersCount:= AcadDocument.Layers.Count;
    for i:= 0 to LayersCount - 1 do
    begin
      Layer:= AcadDocument.Layers.Item(i);
      if AnsiCompareText(Layer.Name, AName) = 0 then Break;
    end;
    AcadDocument.ActiveLayer:= Layer;
  end;
end;

function Point3D(X, Y: Double; Z: Double = 0): TPoint3D;
begin
  Result.X:= X;
  Result.Y:= Y;
  Result.Z:= Z;
end;

function AcadPoint(X, Y: Double; Z: Double = 0): OleVariant;
begin
  Result:= VarArrayCreate([0, 2], varDouble);
  Result[0]:= X;
  Result[1]:= Y;
  Result[2]:= Z;
end;

function AcadPoint(APoint: TPoint3D): OleVariant; overload;
begin
  Result:= VarArrayCreate([0, 2], varDouble);
  Result[0]:= APoint.X;
  Result[1]:= APoint.Y;
  Result[2]:= APoint.Z;
end;

function AcadColorToColor(AColor: AcColor): TColor;
begin
  if AColor in [(Low(AcColorTable) + 1)..High(AcColorTable)] then
    Result:= AcColorTable[AColor - 1]
  else
    Result:= clWhite;
end;

end.
 